--{{{ Libraries

local L = AceLibrary("AceLocale-2.2"):new("GridStatusAurasExt")

--}}}

GridStatusAurasExt = GridStatus:NewModule("GridStatusAurasExt")
GridStatusAurasExt.menuName = L["AurasExt"]


--{{{ AceDB defaults

GridStatusAurasExt.defaultDB = {
	auraGroups = {},
}

--}}}

GridStatusAurasExt.options = false

local auraList = {}
local groupList = {}
local auraTypes = {
    Magic = {},
    Disease = {}, 
    Poison = {}, 
    Curse = {},
}

local MAGIC_TYPE_COLORS = {
    none = {r = 0.5, g = 0.5, b = 0.5, a = 1.0},
    Magic = {r = 0.2, g = 0.4, b = 1, a = 1.0},
    Disease = {r = 0.6, g = 0.4, b = 0, a = 1.0},
    Poison = {r = 0, g = 0.6, b = 0, a = 1.0},
    Curse = {r = 0.8, g = 0, b = 1, a = 1.0},
}

local function addOptionsForAuraTypes(opts, status)
    for auraType, _ in pairs(auraTypes) do
        opts[auraType] = {
            type = "toggle",
            name = (L["Show Debuff type %s"]):format(L[auraType]),
            desc = (L["Enables this status for all debuffs of type %s."]):format(L[auraType]),
            get = function()
                return GridStatusAurasExt.db.profile[status].debuffTypes[auraType]
                end,
            set = function(v)
                GridStatusAurasExt:ShowDebuffTypeForStatus(v, auraType, status)
            end,
            order = 146,
        }
    end
end

local function addOptionsForSettings(opts, status, settings)
    for i, setting in ipairs(settings) do
        local set
        if setting == "onlymine" then
            set = {
                type = "toggle",
                name = L["Only mine"],
                desc = L["Activates this aura group only for auras cast by you."],
                get = function()
                    return GridStatusAurasExt.db.profile[status].onlyMine
                    end,
                set = function(v)
                    GridStatusAurasExt.db.profile[status].onlyMine = v
                    if v and GridStatusAurasExt.db.profile[status].onlyTheirs then
                        GridStatusAurasExt.db.profile[status].onlyTheirs = false
                    end
                    GridStatusAurasExt:UpdateAllUnitAuras()
                end,
                order = 142,
            }
        elseif setting == "onlytheirs" then
            set = {
                type = "toggle",
                name = L["Only theirs"],
                desc = L["Activates this aura group only for auras not cast by you."],
                get = function()
                    return GridStatusAurasExt.db.profile[status].onlyTheirs
                    end,
                set = function(v)
                    GridStatusAurasExt.db.profile[status].onlyTheirs = v
                    if v and GridStatusAurasExt.db.profile[status].onlyMine then
                        GridStatusAurasExt.db.profile[status].onlyMine = false
                    end
                    GridStatusAurasExt:UpdateAllUnitAuras()
                end,
                order = 143,
            }
        elseif setting == "onlybuffs" then
            set = {
                type = "toggle",
                name = L["Only buffs"],
                desc = L["Activates this aura group only for buffs."],
                get = function()
                    return GridStatusAurasExt.db.profile[status].onlyBuffs
                    end,
                set = function(v)
                    GridStatusAurasExt.db.profile[status].onlyBuffs = v
                    if v and GridStatusAurasExt.db.profile[status].onlyDebuffs then
                        GridStatusAurasExt.db.profile[status].onlyDebuffs = false
                    end
                    GridStatusAurasExt:UpdateAllUnitAuras()
                end,
                order = 144,
            }
        elseif setting == "onlydebuffs" then
            set = {
                type = "toggle",
                name = L["Only debuffs"],
                desc = L["Activates this aura group only for debuffs."],
                get = function()
                    return GridStatusAurasExt.db.profile[status].onlyDebuffs
                    end,
                set = function(v)
                    GridStatusAurasExt.db.profile[status].onlyDebuffs = v
                    if v and GridStatusAurasExt.db.profile[status].onlyBuffs then
                        GridStatusAurasExt.db.profile[status].onlyBuffs = false
                    end
                    GridStatusAurasExt:UpdateAllUnitAuras()
                end,
                order = 145,
            }
        elseif setting == "missing" then
            set = {
                type = "toggle",
                name = L["All missing"],
                desc = L["Activates this aura group only if all of its auras are missing."],
                get = function()
                    return GridStatusAurasExt.db.profile[status].allMissing
                    end,
                set = function(v)
                    GridStatusAurasExt.db.profile[status].allMissing = v
                    GridStatusAurasExt:UpdateAllUnitAuras()
                end,
                order = 146,
            }
        elseif setting == "exclude" then
            set = {
                type = "toggle",
                name = L["Exclude auras"],
                desc = L["This option tells this group to use auras added by name or id as a blacklist (excludes them from being shown). Makes only sense together with showing debuffs by types."],
                get = function()
                    return GridStatusAurasExt.db.profile[status].asBlackList
                    end,
                set = function(v)
                    GridStatusAurasExt.db.profile[status].asBlackList = v
                    GridStatusAurasExt:UpdateAllUnitAuras()
                end,
                order = 147,
            }
        elseif setting == "addbyid" then
            set = {
                type = "text",
                name = L["New aura by ID"],
                desc = L["Adds a new Aura with the passed ID to this group"],
                get = function() return "" end,
                set = function(v) GridStatusAurasExt:NewAuraIDToGroup(status, v) end,
                usage = L["<spell id>"],
                order = 150,
            }
        elseif setting == "addbyname" then
            set = {
                type = "text",
                name = L["New aura by name"],
                desc = L["Adds a new Aura with the passed name to this group"],
                get = function() return "" end,
                set = function(v) GridStatusAurasExt:NewAuraNameToGroup(status, v) end,
                usage = L["<aura name>"],
                order = 151,
            }
        elseif setting == "colorbytype" then
            set = {
                type = "toggle",
                name = L["Color by magic type"],
                desc = L["Colors the status by the magic type of the displayed aura. You can edit the colors under \"Grid->Status->Colors\""],
                get = function()
                    return GridStatusAurasExt.db.profile[status].colorByType
                    end,
                set = function(v)
                    GridStatusAurasExt.db.profile[status].colorByType = v
                    GridStatusAurasExt:UpdateAllUnitAuras()
                end,
                order = 148,
            }
        end
        opts[setting] = set
    end
end

local templates = {
    ["none"] = {
        name = L["None"],
        desc = L["Use no template at all, shows all options and pre sets nothing."],
        getOpts = function(status) 
            local opts = {}                         
            addOptionsForSettings(opts, status, {
                "onlymine","onlytheirs",
                "onlybuffs","onlydebuffs",
                "missing",
                "exclude",
                "addbyid","addbyname",
                "colorbytype"})            
            addOptionsForAuraTypes(opts, status)            
            return opts;
        end,
            
    },
    ["bossdebuffs"] = {
        name = L["Simple aura list"],
        desc = L["Use this template if you only want to define a simple list of auras which should be shown in grid (e.g. a list of raid debuffs)."], 
        consts = function(s)
            s.allMissing = false
            s.onlyTheirs = false
            s.onlyMine = false
            s.asBlackList = false
            for debuffType,_ in pairs(auraTypes) do
                s.debuffTypes[debuffType] = false
            end
        end,
        getOpts = function(status) 
            local opts = {}
            addOptionsForSettings(opts, status, {"addbyid","addbyname","onlybuffs","onlydebuffs"})
            return opts
        end,
    },
    ["missing buffs"] = {
        name = L["Missing buffs"],
        desc = L["Use this template for a status which is shown if a certain buff group is missing (like \"Powerword: Fortitude\" and \"Prayer of Fortitude\")"],
        consts = function(s)
            s.allMissing = true
            s.onlyTheirs = false
            s.onlyMine = false
            s.asBlackList = false
            s.onlyBuffs = true
            s.onlyDebuffs = false
            for debuffType,_ in pairs(auraTypes) do
                s.debuffTypes[debuffType] = false
            end
        end,
        getOpts = function(status)  
            local opts = {}
            addOptionsForSettings(opts, status, {"addbyid","addbyname"})
            return opts
        end,
    },
    ["debuffTypes"] = {
        name = L["Debuff types"],
        desc = L["Use this template if you want a status shown on certain dispellable debuff types (Magic, Poisen, etc) where you can exclude certain debuffs from."],
        consts = function(s)
            s.allMissing = false
            s.onlyTheirs = false
            s.onlyMine = false
            s.asBlackList = true
            s.onlyBuffs = false
            s.onlyDebuffs = true
        end,
        getOpts = function(status)
            local opts = {
                ["addbyid"] = {
                    type = "text",
                    name = L["Exclude aura by ID"],
                    desc = L["Excludes the Aura with the passed ID from this group"],
                    get = function() return "" end,
                    set = function(v) GridStatusAurasExt:NewAuraIDToGroup(status, v) end,
                    usage = L["<spell id>"],
                    order = 150,
                },
                ["addbyname"] = {
                    type = "text",
                    name = L["Exclude aura by name"],
                    desc = L["Excludes the Aura with the passed name from this group"],
                    get = function() return "" end,
                    set = function(v) GridStatusAurasExt:NewAuraNameToGroup(status, v) end,
                    usage = L["<aura name>"],
                    order = 151,
                },
            } 
            addOptionsForSettings(opts, status, {"colorbytype"})
            addOptionsForAuraTypes(opts, status)
            
            return opts;
        end,
    },
}

local options = {
    type = "group",
    name = L["AurasExt"],
    desc = L["General option for Auras Ext."],
    args = {
        ["newgroup"] = {
            type = "text",
            name = L["New aura group"],
            desc = L["Creates a new status for a group of auras."],
            get = function ()
                return ""
                end,
            set = function(v)
                GridStatusAurasExt:NewAuraGroup(v)
            end,
            order = 10,
            usage = "",
        },
        ["remove"] = {
            type = "group",
            name = L["Remove group"],
            desc = L["Removes an aura group."],
            args = {},           
        },
    },
}

Grid.options.args["aurasext"] = options

GridStatus.options.args.color.args["magictypes"] = {
    type = "group",
    name = L["Magic type colors"],
    desc = L["Color for magic types (poison, magic, etc)."],
    args = {
        ["none"] = {
            type = "color",
            name = L["No/unknown magic type"],
            desc = L["The color for things with no/unknown magic type."],
            order = 100,
            get = function ()
                      local c = GridStatusAurasExt.db.profile.magicColors["none"]
                      return c.r, c.g, c.b, c.a
                  end,
            set = function (r, g, b, a)
                      local c = GridStatusAurasExt.db.profile.magicColors["none"]
                      c.r, c.g, c.b, c.a = r, g, b, a
                      GridStatus:TriggerEvent("Grid_ColorsChanged")
                  end,
            hasAlpha = true,
        },
    },
}

local settings



function GridStatusAurasExt:OnInitialize()

	self.super.OnInitialize(self)
    settings = GridStatusAurasExt.db.profile
    
    for name, data in pairs(templates) do
        if name ~= "none" then
            Grid.options.args["aurasext"].args["new"..name] = {
                type = "text",
                name = string.format(L["New %s template"], data.name),
                desc = string.format(L["Creates a new status for a group of auras using the %s template. %s"], data.name, data.desc),
                get = function ()
                    return ""
                    end,
                set = function(v)
                    GridStatusAurasExt:NewAuraGroup(v, name)
                end,
                order = 10,
                usage = "",
            }
        end
    end
    
    if not settings.magicColors then
        settings.magicColors = MAGIC_TYPE_COLORS
    end
    MAGIC_TYPE_COLORS = settings.magicColors
    
    for magic, data in pairs(MAGIC_TYPE_COLORS) do
        if magic ~= "none" then
            GridStatus.options.args.color.args["magictypes"].args[magic] = {
                type = "color",
                name = L[magic],
                desc = string.format(L["The color for magic type %s."], L[magic]),
                order = 100,
                get = function ()
                          local c = GridStatusAurasExt.db.profile.magicColors[magic]
                          return c.r, c.g, c.b, c.a
                      end,
                set = function (r, g, b, a)
                          local c = GridStatusAurasExt.db.profile.magicColors[magic]
                          c.r, c.g, c.b, c.a = r, g, b, a
                          GridStatus:TriggerEvent("Grid_ColorsChanged")
                      end,
                hasAlpha = true,
            }
        end
    end
    
    for name, data in pairs(settings.auraGroups) do
        self:RegisterAuraGroup(name, data)
    end    
end

function GridStatusAurasExt:OnEnable()
	--self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    self:RegisterEvent("UNIT_AURA")
    self:RegisterEvent("Grid_UnitJoined")
    self:RegisterEvent("Grid_ColorsChanged")
    
    self:UpdateAllUnitAuras()
end

function GridStatusAurasExt:Grid_UnitJoined(guid, unitid)
	self:UNIT_AURA(unitid)
end

function GridStatusAurasExt:Grid_ColorsChanged()
	self:UpdateAllUnitAuras()
end

function GridStatusAurasExt:SetAuraGroupTemplate(status, template)
    local prevTemplate = settings[status].template
    
    local opts = GridStatus.options.args[status].args
    
    if prevTemplate then
        local templateOpts = templates[prevTemplate].getOpts(status)
        for optName, optData in pairs(templateOpts) do
            opts[optName] = nil
        end
    end
    
    local templateOpts = templates[template].getOpts(status)
    for optName, optData in pairs(templateOpts) do
        opts[optName] = optData
    end
    
    if templates[template].consts then
        templates[template].consts(settings[status])
    end
    
    settings[status].template = template
    
    self:UpdateAllUnitAuras()
end

function GridStatusAurasExt:OnStatusEnable(status)
    local sets = settings[status]
    for id, _ in pairs(sets.ids) do
        self:RegisterId(status, id)
    end
    
    for aura, _ in pairs(sets.names) do
        self:RegisterName(status, aura)
    end
    
    groupList[status] = {settings = sets}
    for debuffType, v in pairs(sets.debuffTypes) do
        if v then
            auraTypes[debuffType][status] = sets
        end
    end
    
    self:UpdateAllUnitAuras()
end

function GridStatusAurasExt:OnStatusDisable(status)
    local sets = settings[status]
    for id, _ in pairs(sets.ids) do
        self:UnregisterId(status, id)
    end
    
    for aura, _ in pairs(sets.names) do
        self:UnregisterName(status, aura)
    end
    
    groupList[status] = nil
    for debuffType, v in pairs(sets.debuffTypes) do
        if v then
            auraTypes[debuffType][status] = nil
        end
    end
    
    for guid, unitid in GridRoster:IterateRoster() do
        self.core:SendStatusLost(guid, status)
    end
end

function GridStatusAurasExt:RemoveAuraGroup(name) 
    local status = "status_"..name
    self:UnregisterStatus(status)
    
    local sets = settings[status]
    for id, _ in pairs(sets.ids) do
        self:UnregisterId(status, id)
    end
    
    for aura, _ in pairs(sets.names) do
        self:UnregisterName(status, aura)
    end
    
    groupList[status] = nil
    settings[status] = nil
    options.args.remove.args[status] = nil
    
    settings.auraGroups[name] = nil
end

function GridStatusAurasExt:RegisterAuraGroup(name, data, template)
    local status = "status_"..name
    
    local sets = settings[status]
    if not sets then
        sets = {
            enable = true, 
            color = { r = 1, g = 1, b = 1, a = 1 }, 
            priority = 50,  
            allMissing = false,
            onlyTheirs = false,
            onlyMine = false,
            onlyBuffs = false,
            onlyDebuffs = false,
            asBlackList = false,
            colorByType = false,
            ids = {},
            names = {},
            template = template,
            debuffTypes = {},
        }
        settings[status] = sets
    else
        if not sets.debuffTypes then sets.debuffTypes = {} end
    end
    
    options.args.remove.args[status] = {
        type = "execute",
        name = name.." - RELOAD UI!",
        desc = L["Removes this aura group. Please Reload your UI after that!"],        
        func = function() self:RemoveAuraGroup(name) end,
        order = 1,
    }
    
    local opts = {
        ["remove"] = {
            type = "group",
            name = L["Remove"],
            desc = L["Remove an aura from the group."],
            args = {},
        },
        ["templates"] = {
            type = "group",
            name = L["Templates"],
            desc = L["Use a certain settings template for this aura group."],
            args = {},
        },        
    }
    
    for templateName, data in pairs(templates) do
        local order = (templateName == "none") and 1 or 2
        opts.templates.args[templateName] = {
            type = "execute",
            name = data.name,
            desc = data.desc,        
            func = function() self:SetAuraGroupTemplate(status, templateName) end,
            order = order,
        }
    end
    
    self:RegisterStatus(status, name, opts, true)
    
    self:SetAuraGroupTemplate(status, sets.template or "none")
    
    if sets.enable then
        for id, _ in pairs(sets.ids) do
            self:RegisterId(status, id)
        end
        
        for aura, _ in pairs(sets.names) do
            self:RegisterName(status, aura)
        end
        
        groupList[status] = {settings = sets}
        for debuffType, v in pairs(sets.debuffTypes) do
            if v then
                auraTypes[debuffType][status] = sets
            end
        end
    end
end

function GridStatusAurasExt:UnregisterName(status, aura)
    auraList[aura][status] = nil
end

function GridStatusAurasExt:RegisterName(status, aura)
    
    local auraData = auraList[aura]
    if not auraData then
        auraData = {}
        auraList[aura] = auraData
    end
        
    if settings[status].names[aura] then
        auraData[status] = {settings = settings[status]}    
    end
    
    local opts = GridStatus.options.args[status].args
    opts[aura] = {
        type = "toggle",
        name = aura,
        desc = aura,
        get = function()
            return settings[status].names[aura]
            end,
        set = function(v)
            settings[status].names[aura] = v
            if v then
                auraList[aura][status] = {settings = settings[status]}
            else
                auraList[aura][status] = nil
            end
            self:UpdateAllUnitAuras()
        end,
        order = 160,
    }
    opts.remove.args[aura] = {
        type = "execute",
        name = aura,
        desc = L["Removes this aura."],        
        func = function() self:RemoveNameFromGroup(status, aura) end,
        order = 1,
    }
end

function GridStatusAurasExt:ShowDebuffTypeForStatus(show, debuffType, status)
    settings[status].debuffTypes[debuffType] = show
    
    if show then
        auraTypes[debuffType][status] = settings[status]
    else
        auraTypes[debuffType][status] = nil
    end
    
    self:UpdateAllUnitAuras()
end

function GridStatusAurasExt:UnregisterId(status, id)
    local aura, rank, icon = GetSpellInfo(id)
    auraList[aura][status] = nil
end

function GridStatusAurasExt:RegisterId(status, id)
    local aura, rank, icon = GetSpellInfo(id)
    local auraData = auraList[aura]
    if not auraData then
        auraData = {}
        auraList[aura] = auraData
    end
    
    if settings[status].names[aura] then
        auraData[status] = {
            settings = settings[status], 
            icon = icon,
            rank = rank,
        }
    end

    local opts = GridStatus.options.args[status].args
    
    opts[id] = {
        type = "toggle",
        name = GetSpellInfo(id).."("..id..")",
        desc = id,
        get = function()
            return settings[status].ids[id]
            end,
        set = function(v)
            settings[status].ids[id] = v
            if v then
                auraList[aura][status] = {
                    settings = settings[status], 
                    icon = icon,
                    rank = rank,
                }
            else
                auraList[aura][status] = nil
            end
            self:UpdateAllUnitAuras()
        end,
        order = 160,
    }
    opts.remove.args[id] = {
        type = "execute",
        name = GetSpellInfo(id).."("..id..")",
        desc = L["Removes this aura."],        
        func = function() self:RemoveIdFromGroup(status, id) end,
        order = 1,
    }
end

function GridStatusAurasExt:RemoveIdFromGroup(status, id)
    local opts = GridStatus.options.args[status].args
    opts[id] = nil
    opts.remove.args[id] = nil
    settings[status].ids[id] = nil
    
    local aura = GetSpellInfo(id)
    auraList[aura][status] = nil
    
    self:UpdateAllUnitAuras()
end

function GridStatusAurasExt:RemoveNameFromGroup(status, aura)
    local opts = GridStatus.options.args[status].args
    opts[aura] = nil
    opts.remove.args[aura] = nil
    settings[status].names[aura] = nil
    auraList[aura][status] = nil
    self:UpdateAllUnitAuras()
end

function GridStatusAurasExt:NewAuraIDToGroup(status, id)
    settings[status].ids[id] = true
    self:RegisterId(status, id)    
    self:UpdateAllUnitAuras()
end

function GridStatusAurasExt:NewAuraNameToGroup(status, aura)
    settings[status].names[aura] = true
    self:RegisterName(status, aura)    
    self:UpdateAllUnitAuras()
end

function GridStatusAurasExt:NewAuraGroup(name, template)   
    settings.auraGroups[name] = true
    self:RegisterAuraGroup(name, settings.auraGroups[name], template)
end

function GridStatusAurasExt:UpdateAllUnitAuras()
	for guid, unitid in GridRoster:IterateRoster() do
		self:UNIT_AURA(unitid)
	end
end




function GridStatusAurasExt:UNIT_AURA(unit)
    local guid = UnitGUID(unit)
	if not GridRoster:IsGUIDInRaid(guid) then
		return
	end
    
    for status, data in pairs(groupList) do
        data.active = false
    end
    
    local i = 1
    local name, rank, icon, count, debuffType, duration, expirationTime, source, isStealable = UnitDebuff(unit, i)
    while name do
        local aura = auraList[name]
        if aura then
            local isMine = source and UnitIsUnit(source, "player")
            for status, data in pairs(aura) do
                if not groupList[status].active
                   and not data.settings.asBlackList
                   and (not data.settings.onlyBuffs) 
                   and (not data.icon or data.icon == icon) 
                   and (not data.rank or data.rank == rank)
                   and (not data.settings.onlyMine or isMine)
                   and (not data.settings.onlyTheirs or not isMine) then
                        
                    groupList[status].active = true
                    groupList[status].settings.name = name
                    groupList[status].settings.icon = icon
                    groupList[status].start = expirationTime - duration
                    groupList[status].duration = duration
                    groupList[status].count = count
                    groupList[status].magicType = debuffType or "none"
                end
            end
        end
        if debuffType then
            local typedGroups = auraTypes[debuffType]
            for status, data in pairs(typedGroups) do
                if not groupList[status].active 
                   and (not data.asBlackList or not aura or not aura[status]) then
                    groupList[status].active = true
                    groupList[status].settings.name = name
                    groupList[status].settings.icon = icon
                    groupList[status].start = expirationTime - duration
                    groupList[status].duration = duration
                    groupList[status].count = count
                    groupList[status].magicType = debuffType or "none"
                end
            end
        end
        i = i + 1
        name, rank, icon, count, debuffType, duration, expirationTime, source, isStealable = UnitDebuff(unit, i)
    end
    
    local i = 1
    local name, rank, icon, count, buffType, duration, expirationTime, source, isStealable = UnitBuff(unit, i)
    while name do
        local aura = auraList[name]
        if aura then
            local isMine = source and UnitIsUnit(source, "player")
            for status, data in pairs(aura) do
                if  not groupList[status].active
                    and (not data.settings.onlyDebuffs) 
                    and (not data.icon or data.icon == icon) 
                    and (not data.rank or data.rank == rank)
                    and (not data.settings.onlyMine or isMine)
                    and (not data.settings.onlyTheirs or not isMine) then
                    
                    groupList[status].active = true
                    groupList[status].settings.name = name
                    groupList[status].settings.icon = icon
                    groupList[status].start = expirationTime - duration
                    groupList[status].duration = duration
                    groupList[status].count = count
                    groupList[status].magicType = buffType or "none"
                end
            end        
        end
        i = i + 1
        name, rank, icon, count, buffType, duration, expirationTime, source, isStealable = UnitBuff(unit, i)
    end
    
    for status, data in pairs(groupList) do
        if data.active == (not data.settings.allMissing) then
            --print(status.." "..(data.settings.name or "nil"))
            self.core:SendStatusGained(guid,
                           status,
                           data.settings.priority,
                           (data.settings.range and 40),
                           (data.settings.colorByType and MAGIC_TYPE_COLORS[data.magicType]) or data.settings.color,
                           data.settings.name,
                           data.count,
                           nil,
                           data.settings.icon,
                           (data.active and data.start) or nil,
                           (data.active and data.duration) or nil,
                           (data.active and data.count) or nil)
        else
            --print(status.." lost")
            self.core:SendStatusLost(guid, status)
        end
    end
end



function GridStatusAurasExt:BruteForceAuraData(unit, id, aura, isMine, isTheirs, buffsOnly, debuffOnly)
    assert(not(isMine and isTheirs))
    assert(not(buffsOnly and debuffOnly))
    
    local idIcon
    if id then
        _, _, idIcon = GetSpellInfo(id)
    end
    
    if not buffsOnly then
        for i=1,40 do
            local name, rank, icon, count, debuffType, duration, expirationTime, source, isStealable = UnitDebuff(unit, i)
            if name then
                print(name)
                local mine = UnitIsUnit(source, "player")
                if name == aura and (not id or icon == idIcon) and (not isMine or mine) and (not isTheirs or not mine) then
                    return icon, count, duration, expirationTime
                end
            end
        end
    end
    
    if not debuffsOnly then
        for i=1,40 do
            local name, rank, icon, count, debuffType, duration, expirationTime, source, isStealable = UnitBuff(unit, i)
            if name then
                print(name)
                local mine = source and UnitIsUnit(source, "player")
                if name == aura and (not id or icon == idIcon) and (not isMine or mine) and (not isTheirs or not mine) then
                    return icon, count, duration, expirationTime
                end
            end
        end
    end
end 


function GridStatusAurasExt:SendAura(group, spell, guid, icon, count, duration, expirationTime)
    local status = "status_"..group
    local grpSets = settings[status]
    print(guid)
    
    local start = nil
    if duration and expirationTime then
        start = expirationTime - duration
    end
    
    self.core:SendStatusGained(guid,
                           status,
                           grpSets.priority,
                           (grpSets.range and 40),
                           grpSets.color,
                           spell,
                           count,
                           nil,
                           icon,
                           start,
                           duration,
                           count)
end

function GridStatusAurasExt:LostAura(group, guid)
    self.core:SendStatusLost(guid, "status_"..group)
end

function GridStatusAurasExt:AuraApplied(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, auraType)  
    spellId = tostring(spellId)
    for grp, grpData in pairs(settings.auraGroups) do
        if grpData.ids[spellId] or grpData.names[spellName] then
            print("isaura")
            local icon, count, duration, expTime = self:BruteForceAuraData(dstName, spellId, spellName, grpData.onlyMine, grpData.onlyTheirs)
            if icon then 
                grpData.icon = icon
                local auras = grpAuras[grp]
                
                if not auras[spellName] then
                    auras[spellName] = {count = 0}
                end
                if not auras[spellName][spellId] then
                    auras[spellName].count = auras[spellName].count + 1
                    auras.count = auras.count + 1
                end
                auras[spellName][spellId] = true
                
                if grpData.allMissing then
                    self:LostAura(grp, dstGUID)
                else
                    self:SendAura(grp, spellName, dstGUID, icon, count, duration, expTime)
                end
            end
        end
    end
end

function GridStatusAurasExt:AuraRefresh(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, auraType)    
    assert(false)
end

function GridStatusAurasExt:AuraRemoved(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, auraType)  
    spellId = tostring(spellId)
    for grp, grpData in pairs(settings.auraGroups) do     
        local auras = grpAuras[grp][spellName]
        if auras and auras[spellId] and (grpData.ids[spellId] or grpData.names[spellName]) then
            local icon, count, duration, expTime = self:BruteForceAuraData(dstName, spellId, spellName, grpData.onlyMine, grpData.onlyTheirs)
            
            if icon then     
                if grpData.allMissing then
                    self:LostAura(grp, dstGUID)
                else
                    self:SendAura(grp, spellName, dstGUID, icon, count, duration, expTime)
                end
            else                
                auras[spellId] = nil
                auras.count = auras.count - 1
                grpAuras[grp].count = grpAuras[grp].count - 1
                

                if grpAuras[grp].count <= 0 then
                    assert(grpAuras[grp].count == 0)
                    local status = "status_"..grp
                    if grpData.allMissing then
                        self:SendAura(grp, spellName, dstGUID, grpData.icon)
                    else
                        self:LostAura(grp, dstGUID)
                    end
                else
                    for name, auras in pairs(grpAuras[grp]) do
                        for id, _ in pairs(auras) do
                            if name ~= "count" then
                                icon, count, duration, expTime = self:BruteForceAuraData(dstName, id, name, grpData.onlyMine, grpData.onlyTheirs)
                                assert(icon)
                                if grpData.allMissing then
                                    self:LostAura(grp, dstGUID)
                                else
                                    self:SendAura(grp, name, dstGUID, icon, count, duration, expTime)
                                end
                                break
                            end
                        end
                    end
                end
            end
        end
    end
end


local EventParse = {}
EventParse["SPELL_AURA_APPLIED"] = GridStatusAurasExt.AuraApplied
EventParse["SPELL_AURA_REFRESH"] = GridStatusAurasExt.AuraApplied
EventParse["SPELL_AURA_REMOVED"] = GridStatusAurasExt.AuraRemoved 
EventParse["SPELL_AURA_BROKEN"] = GridStatusAurasExt.AuraRemoved
EventParse["SPELL_AURA_BROKEN_SPELL"] = GridStatusAurasExt.AuraRemoved

local bit_band = _G.bit.band

local function isInPartyOrRaidOrMe(unitFlags)
    return  bit_band(unitFlags, COMBATLOG_OBJECT_AFFILIATION_RAID) == COMBATLOG_OBJECT_AFFILIATION_RAID or 
            bit_band(unitFlags, COMBATLOG_OBJECT_AFFILIATION_PARTY) == COMBATLOG_OBJECT_AFFILIATION_PARTY or 
            bit_band(unitFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) == COMBATLOG_OBJECT_AFFILIATION_MINE 
end



function GridStatusAurasExt:COMBAT_LOG_EVENT_UNFILTERED(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)   
    if not (isInPartyOrRaidOrMe(dstFlags)) then
        return
    end
	local parsefunc = EventParse[eventtype]
	
    --print("Event: "..eventtype)
	if parsefunc then  
		parsefunc(self, timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	end
end
